package de.dfki.mlt.advent2022;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

class Day02 {

  private static final Logger logger = LoggerFactory.getLogger(Day02.class);

  private enum Shape {
    ROCK,
    PAPER,
    SCISSORS
  }

  private enum Outcome {
    WIN,
    LOSS,
    DRAW
  }

  static Shape getShapeForString(String s) {
    return switch (s) {
      case "A", "X" -> Shape.ROCK;
      case "B", "Y" -> Shape.PAPER;
      case "C", "Z" -> Shape.SCISSORS;
      default -> null;
    };
  }

  static Outcome getOutcomeForString(String s) {
    return switch (s) {
      case "X" -> Outcome.LOSS;
      case "Y" -> Outcome.DRAW;
      case "Z" -> Outcome.WIN;
      default -> null;
    };
  }

  static int getScore(Shape player1Shape, Shape player2Shape) {

    logger.debug("player1: {} player2: {}", player1Shape, player2Shape);

    int shapeScore =
        switch (player2Shape) {
          case ROCK -> 1;
          case PAPER -> 2;
          case SCISSORS -> 3;
        };

    int outcomeScore =
        switch (player1Shape) {
          case ROCK -> {
            yield switch (player2Shape) {
              case ROCK -> 3;
              case PAPER -> 6;
              case SCISSORS -> 0;
            };
          }
          case PAPER -> {
            yield switch (player2Shape) {
              case ROCK -> 0;
              case PAPER -> 3;
              case SCISSORS -> 6;
            };
          }
          case SCISSORS -> {
            yield switch (player2Shape) {
              case ROCK -> 6;
              case PAPER -> 0;
              case SCISSORS -> 3;
            };
          }
        };
    int totalScore = shapeScore + outcomeScore;

    logger.debug("score: {}", totalScore);
    return totalScore;
  }

  static int firstTask(Path path) throws IOException {

    List<String> lines = Files.readAllLines(path);

    int score = 0;
    for (String oneLine : lines) {
      String[] parts = oneLine.split(" ");
      score += getScore(getShapeForString(parts[0]), getShapeForString(parts[1]));
    }

    logger.info("task 1 total score: {}", score);
    return score;
  }

  static Shape getAnswer(Shape player1Shape, Outcome player2) {
    return switch (player2) {
      case LOSS -> {
        yield switch (player1Shape) {
          case ROCK -> Shape.SCISSORS;
          case PAPER -> Shape.ROCK;
          case SCISSORS -> Shape.PAPER;
        };
      }
      case DRAW -> {
        yield switch (player1Shape) {
          case ROCK -> Shape.ROCK;
          case PAPER -> Shape.PAPER;
          case SCISSORS -> Shape.SCISSORS;
        };
      }
      case WIN -> {
        yield switch (player1Shape) {
          case ROCK -> Shape.PAPER;
          case PAPER -> Shape.SCISSORS;
          case SCISSORS -> Shape.ROCK;
        };
      }
    };
  }

  static int secondTask(Path path) throws IOException {

    List<String> lines = Files.readAllLines(path);

    int score = 0;
    for (String oneLine : lines) {
      String[] parts = oneLine.split(" ");
      Shape player1Shape = getShapeForString(parts[0]);
      Shape answer = getAnswer(player1Shape, getOutcomeForString(parts[1]));
      score += getScore(player1Shape, answer);
    }

    logger.info("task 2 total score: {}", score);
    return score;
  }
}
