package de.dfki.mlt.advent2022;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

class Day04 {

  private static final Logger logger = LoggerFactory.getLogger(Day04.class);

  private static Pattern sectionPattern = Pattern.compile("(\\d+)-(\\d+),(\\d+)-(\\d+)");

  static int firstTask(Path path) throws IOException {

    List<String> lines = Files.readAllLines(path);

    int overlapCount = 0;
    for (String oneLine : lines) {
      logger.debug(oneLine);
      Matcher matcher = sectionPattern.matcher(oneLine);
      if (!matcher.matches()) {
        logger.error("no match for {}", oneLine);
        continue;
      }
      int elf1Start = Integer.parseInt(matcher.group(1));
      int elf1End = Integer.parseInt(matcher.group(2));
      int elf2Start = Integer.parseInt(matcher.group(3));
      int elf2End = Integer.parseInt(matcher.group(4));
      if (elf1Start >= elf2Start && elf1End <= elf2End
          || elf2Start >= elf1Start && elf2End <= elf1End) {
        logger.debug("overlapping");
        overlapCount++;
      }
    }

    logger.info("task 1 overlap count: {}", overlapCount);
    return overlapCount;
  }

  static int secondTask(Path path) throws IOException {

    List<String> lines = Files.readAllLines(path);

    int overlapCount = 0;
    for (String oneLine : lines) {
      logger.debug(oneLine);
      Matcher matcher = sectionPattern.matcher(oneLine);
      if (!matcher.matches()) {
        logger.error("no match for {}", oneLine);
        continue;
      }
      int elf1Start = Integer.parseInt(matcher.group(1));
      int elf1End = Integer.parseInt(matcher.group(2));
      int elf2Start = Integer.parseInt(matcher.group(3));
      int elf2End = Integer.parseInt(matcher.group(4));
      if (elf2Start >= elf1Start && elf2Start <= elf1End
          || elf1Start >= elf2Start && elf1Start <= elf2End) {
        overlapCount++;
      }
    }

    logger.info("task 2 overlap count: {}", overlapCount);
    return overlapCount;
  }
}
