package de.dfki.mlt.advent2022;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Stack;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

class Day05 {

  private static final Logger logger = LoggerFactory.getLogger(Day05.class);

  static int initStacks(List<String> lines, List<Stack<Character>> stacks) {

    // collect lines with stack items
    List<String> stackLines = new ArrayList<>();
    int moveStartLine = 0;
    int numberOfStacks = 0;
    for (int i = 0; i < lines.size(); i++) {
      String oneLine = lines.get(i);
      logger.debug("{}: {}", i, oneLine);
      if (oneLine.startsWith(" 1")) {
        // get number of stacks
        String[] parts = oneLine.split("\\s+");
        numberOfStacks = Integer.parseInt(parts[parts.length - 1]);
        logger.debug("number of stacks: {}", numberOfStacks);
        moveStartLine = i + 2;
        break;
      }
      stackLines.add(oneLine);
    }

    // init stacks
    for (int i = 0; i < numberOfStacks; i++) {
      stacks.add(new Stack<>());
    }

    // add items to stacks
    Collections.reverse(stackLines);
    for (String oneStackLine : stackLines) {
      for (int j = 0; j < numberOfStacks; j++) {
        int itemIndex = 1 + (j * 4);
        char item = oneStackLine.charAt(itemIndex);
        if (Character.isLetter(item)) {
          stacks.get(j).push(item);
        }
      }
    }

    logger.debug("original stacks:");
    for (Stack<Character> oneStack : stacks) {
      logger.debug(oneStack + "");
    }

    return moveStartLine;
  }

  static String applyMoves(
      List<String> lines,
      int moveStartLine,
      List<Stack<Character>> stacks,
      boolean crateMover9001) {

    for (int i = moveStartLine; i < lines.size(); i++) {
      String oneMoveLine = lines.get(i);
      String[] parts = oneMoveLine.split(" ");
      logger.debug(oneMoveLine);
      int count = Integer.parseInt(parts[1]);
      int startStackIndex = Integer.parseInt(parts[3]);
      int endStackIndex = Integer.parseInt(parts[5]);
      // index starts at 0, but in moves, it starts at 1
      startStackIndex--;
      endStackIndex--;
      Stack<Character> startStack = stacks.get(startStackIndex);
      Stack<Character> endStack = stacks.get(endStackIndex);

      if (!crateMover9001) {
        // task 1
        for (int j = 0; j < count; j++) {
          endStack.push(startStack.pop());
        }
      } else {
        // task 2
        Stack<Character> tempStack = new Stack<>();
        for (int j = 0; j < count; j++) {
          tempStack.push(startStack.pop());
        }
        for (int j = 0; j < count; j++) {
          endStack.push(tempStack.pop());
        }
      }
    }

    logger.debug("rearranged stacks");
    for (Stack<Character> oneStack : stacks) {
      logger.debug(oneStack + "");
    }

    // create result
    StringBuilder result = new StringBuilder();
    for (Stack<Character> oneStack : stacks) {
      result.append(oneStack.peek());
    }
    return result.toString();
  }

  static String firstTask(Path path) throws IOException {

    List<String> lines = Files.readAllLines(path);

    List<Stack<Character>> stacks = new ArrayList<>();
    int moveStartLine = initStacks(lines, stacks);
    String topItems = applyMoves(lines, moveStartLine, stacks, false);

    logger.info("task 1 top items: {}", topItems);
    return topItems;
  }

  static String secondTask(Path path) throws IOException {

    List<String> lines = Files.readAllLines(path);

    List<Stack<Character>> stacks = new ArrayList<>();
    int moveStartLine = initStacks(lines, stacks);
    String topItems = applyMoves(lines, moveStartLine, stacks, true);

    logger.info("task 2 top items: {}", topItems);
    return topItems;
  }
}
