package de.dfki.mlt.advent2022;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Arrays;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

class Day08 {

  private static final Logger logger = LoggerFactory.getLogger(Day08.class);

  static int[][] initTrees(List<String> lines) {
    int[][] trees = new int[lines.size()][lines.get(0).length()];
    for (int i = 0; i < lines.size(); i++) {
      String oneLine = lines.get(i);
      trees[i] = Arrays.stream(oneLine.split("")).mapToInt(Integer::parseInt).toArray();
    }
    return trees;
  }

  static boolean isVisibleFromLeft(int[][] trees, int i, int j) {
    logger.debug("checking left");
    int treeHeight = trees[i][j];
    for (int k = 0; k < j; k++) {
      logger.debug("{}, {}", i, k);
      if (trees[i][k] >= treeHeight) {
        logger.debug("found higher: {} ({}, {})", trees[i][k], i, k);
        return false;
      }
    }
    return true;
  }

  static boolean isVisibleFromRight(int[][] trees, int i, int j) {
    logger.debug("checking right");
    int treeHeight = trees[i][j];
    for (int k = j + 1; k < trees[i].length; k++) {
      logger.debug("{}, {}", i, k);
      if (trees[i][k] >= treeHeight) {
        logger.debug("found higher: {} ({}, {})", trees[i][k], i, k);
        return false;
      }
    }
    return true;
  }

  static boolean isVisibleFromAbove(int[][] trees, int i, int j) {
    logger.debug("checking above");
    int treeHeight = trees[i][j];
    for (int k = 0; k < i; k++) {
      logger.debug("{}, {}", k, j);
      if (trees[k][j] >= treeHeight) {
        logger.debug("found higher: {} ({}, {})", trees[k][j], k, j);
        return false;
      }
    }
    return true;
  }

  static boolean isVisibleFromBelow(int[][] trees, int i, int j) {
    logger.debug("checking below");
    int treeHeight = trees[i][j];
    for (int k = i + 1; k < trees.length; k++) {
      logger.debug("{}, {}", k, j);
      if (trees[k][j] >= treeHeight) {
        logger.debug("found higher: {} ({}, {})", trees[k][j], k, j);
        return false;
      }
    }
    return true;
  }

  static int firstTask(Path path) throws IOException {

    List<String> lines = Files.readAllLines(path);
    int[][] trees = initTrees(lines);

    int visibleCount = 2 * lines.size() + 2 * (lines.get(0).length() - 2);
    for (int i = 1; i < trees.length - 1; i++) {
      for (int j = 1; j < trees[i].length - 1; j++) {
        logger.debug("checking {} ({}, {})...", trees[i][j], i, j);
        if (isVisibleFromLeft(trees, i, j)
            || isVisibleFromRight(trees, i, j)
            || isVisibleFromAbove(trees, i, j)
            || isVisibleFromBelow(trees, i, j)) {
          visibleCount++;
        }
      }
    }

    logger.info("task 1 visible: {}", visibleCount);
    return visibleCount;
  }

  static int getViewToLeft(int[][] trees, int i, int j) {
    logger.debug("looking left");
    int treeHeight = trees[i][j];
    int count = 0;
    for (int k = j - 1; k >= 0; k--) {
      logger.debug("{}, {}", i, k);
      count++;
      if (trees[i][k] >= treeHeight) {
        logger.debug("found higher: {} ({}, {})", trees[i][k], i, k);
        break;
      }
    }
    return count;
  }

  static int getViewToRight(int[][] trees, int i, int j) {
    logger.debug("looking right");
    int treeHeight = trees[i][j];
    int count = 0;
    for (int k = j + 1; k < trees[i].length; k++) {
      logger.debug("{}, {}", i, k);
      count++;
      if (trees[i][k] >= treeHeight) {
        logger.debug("found higher: {} ({}, {})", trees[i][k], i, k);
        break;
      }
    }
    return count;
  }

  static int getViewToAbove(int[][] trees, int i, int j) {
    logger.debug("looking above");
    int treeHeight = trees[i][j];
    int count = 0;
    for (int k = i - 1; k >= 0; k--) {
      logger.debug("{}, {}", i, k);
      count++;
      if (trees[k][j] >= treeHeight) {
        logger.debug("found higher: {} ({}, {})", trees[k][j], k, j);
        break;
      }
    }
    return count;
  }

  static int getViewToBelow(int[][] trees, int i, int j) {
    logger.debug("looking above");
    int treeHeight = trees[i][j];
    int count = 0;
    for (int k = i + 1; k < trees.length; k++) {
      logger.debug("{}, {}", i, k);
      count++;
      if (trees[k][j] >= treeHeight) {
        logger.debug("found higher: {} ({}, {})", trees[k][j], k, j);
        break;
      }
    }
    return count;
  }

  static int secondTask(Path path) throws IOException {

    List<String> lines = Files.readAllLines(path);
    int[][] trees = initTrees(lines);

    int maxScenicScore = Integer.MIN_VALUE;
    for (int i = 1; i < trees.length - 1; i++) {
      for (int j = 1; j < trees[i].length - 1; j++) {
        logger.debug("checking {} ({}, {})...", trees[i][j], i, j);
        int scenicScore =
            getViewToLeft(trees, i, j)
                * getViewToRight(trees, i, j)
                * getViewToAbove(trees, i, j)
                * getViewToBelow(trees, i, j);
        maxScenicScore = Math.max(maxScenicScore, scenicScore);
      }
    }

    logger.info("task 2 scenic score: {}", maxScenicScore);
    return maxScenicScore;
  }
}
